from abc import ABCMeta, abstractmethod
from typing import Literal, Sequence, Type, TypeVar

from tp_interfaces.abstract import AbstractProcessor, ImmutableBaseModel


class EntityModel(ImmutableBaseModel):
    type: Literal["document", "concept"]
    id: str


class S3Result(ImmutableBaseModel):
    bucket: str
    path: str


class ExporterResult(ImmutableBaseModel):
    message: str
    result: S3Result | None


_Config = TypeVar('_Config', bound=ImmutableBaseModel)


class AbstractExporter(AbstractProcessor[_Config, EntityModel, ExporterResult], metaclass=ABCMeta):
    async def process_doc(self, document: EntityModel, config: _Config) -> ExporterResult:
        return await self.process_docs([document], config)

    @abstractmethod
    async def process_docs(self, documents: Sequence[EntityModel], config: _Config) -> ExporterResult:
        pass

    @property
    def input_type(self) -> Type[EntityModel]:
        return EntityModel

    @property
    def output_type(self) -> Type[ExporterResult]:
        return ExporterResult



from abc import ABCMeta, abstractmethod
from os import PathLike
from pathlib import Path
from typing import Generic, Iterable, TypeVar

from fsspec.implementations.local import LocalFileSystem
from fsspec.spec import AbstractFileSystem

from .split_data import AbstractSplitData

_ElementType = TypeVar('_ElementType')


class AbstractDatasetManager(Generic[_ElementType], metaclass=ABCMeta):

    def __init__(self, path: str | PathLike, file_system: AbstractFileSystem | None = None):
        self._fs = file_system if file_system else LocalFileSystem()
        self._path = Path(path)
        self._extra_data = None

    @abstractmethod
    def add(self, element: _ElementType) -> None:
        ...

    def update(self, elements: Iterable[_ElementType]) -> None:
        for element in elements:
            self.add(element)

    def add_extra_info(self, **kwargs):
        if kwargs:
            self._extra_data = kwargs

    @abstractmethod
    def remove(self, element: _ElementType) -> None:
        ...

    @abstractmethod
    def save(self, version: str | None = None, exist_ok: bool = False) -> None:
        # TODO: reconsider the `version` parameter in the interface.
        #  It might be better to remove it from the interface and handle it via an implementation-level setter,
        #  since not all managers support the concept of versions.
        ...

    @abstractmethod
    def get_dataset(self) -> AbstractSplitData:
        ...



from abc import ABCMeta, abstractmethod
from typing import Generic, Iterable, TypeVar

_ItemType = TypeVar('_ItemType')


class AbstractSplitData(Generic[_ItemType], metaclass=ABCMeta):

    @property
    @abstractmethod
    def roles(self) -> set[str]:
        ...

    @abstractmethod
    def get_data(self, role: str | None = None) -> Iterable[_ItemType]:
        ...
